unit Unit1;

// [The "BSD licence"]
//
//  Copyright (c) 2013 Bruno Verhue
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//  1. Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright
//     notice, this list of conditions and the following disclaimer in the
//     documentation and/or other materials provided with the distribution.
//  3. The name of the author may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
//  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
//  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
//  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// A example application to load some SVG Inmages and move them around a
// zoom/pan layout.

// When loading the "animatied clock", it will have some interactive
// functionality: it will be set and updated to the current time, and you
// can move the clock hands by mouse.

// When loading SVG Images the SVG Agent will parse the with the property
// "Parenting" set to True, but the applications should also work when set
// to False.

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
{$IF Defined(VER240) or Defined(VER250)}
  System.UIConsts,
{$IFEND}
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.Objects, FMX.Layouts,
  FMX.Memo, FMX.Ani, System.Contnrs, FMX.Edit, FMX.Effects, FMX.Menus, FMX.TreeView,
  FMX.StdCtrls, FMX.ListBox,
  Xml.xmldom, Xml.XMLIntf, Xml.XMLDoc, Xml.adomxmldom,
{$IFDEF MSWINDOWS}
  Xml.Win.msxmldom,
{$ENDIF}
  BVE.SVGControl,
  //BVE.SVGXMLWrapperNativeXML,
  //BVE.SVGXMLWrapperFPC;
  BVE.SVGXMLWrapperDelphi;

type
  // Class for moving selected objects with mouse
  TSVGSelection = class(TLayout)
  private
    FSelList : TObjectList;
  protected
    function  GetActive: boolean;
    procedure SetActive( const aValue : boolean);
  public
    constructor Create( AOwner: TComponent); override;
    destructor  Destroy; override;

    procedure SelectObject( aSVGGroup : TSVGGroup);
    procedure AddToSelection( aSVGGroup : TSVGGroup);
    procedure UnSelectAll;
    function  Selected( aSVGGroup : TSVGGroup): boolean;

    property Active : boolean read GetActive write SetActive;
  end;

  // Class for displaying structure of SVG object
  TSVGTreeViewItem = class(TTreeViewItem)
  private
    FSVGObject : TSVGObject;
    FSVGParent : TSVGGroup;
    procedure SetSVGText;
  protected
    function GetObjectVisible: boolean;
    procedure SetObjectVisible( aValue : boolean);
    procedure SetSVGObject( aValue : TSVGObject);
  public
    constructor Create( AOwner: TComponent); override;
    destructor  Destroy; override;

    property SVGParent : TSVGGroup read FSVGParent write FSVGParent;
    property SVGObject : TSVGObject read FSVGObject write SetSVGObject;
    property ObjectVisible : boolean read GetObjectVisible write SetObjectVisible;
  end;

  // Example of interactive SVG
  TSVGAnimatedClock = class(TSVGControl)
  private
    FGearSeconds : TSVGGroup;
    FGear7_5 : TSVGGroup;
    FGearMinutes : TSVGGroup;
    FGear180 : TSVGGroup;
    FGearHours : TSVGGroup;

    FHandSecond : TSVGGroup;
    FPathHandSecond : TSVGPath;

    FHandMinute : TSVGGroup;
    FPathHandMinute : TSVGPath;

    FHandHour : TSVGGroup;
    FPathhandHour : TSVGPath;

    FHour, FMin, FSec : single;
    FTimer : TTimer;

    FMinuteDY : single;
    procedure SetLayoutAngle( aLayout : TSVGGroup; aAngle : single);
  public
    constructor Create( AOwner: TComponent); override;
    constructor Create( AOwner: TComponent; aId : string; aSVGParent : TSVGGroup; aCTM, aUserMatrix : TMatrix); override;
    destructor  Destroy; override;

    function    CreateCopy( AOwner : TComponent; aSVGParent : TSVGGroup): TSVGObject; override;
    procedure   Assign(Source: TPersistent); override;

    procedure   ProcessMouseMove( aSVGObject: TSVGObject; Shift: TShiftState; dX, dY: Single);
    procedure   HandMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Single);

    procedure   AddDeltaSec( dm : single);
    procedure   AddDeltaMin( dm : single);
    procedure   AddDeltaHour( dm : single);
    procedure   Update;
    procedure   Init;
    procedure   SetTime( Sender : TObject);

  end;

  TfrmSVGTest = class(TForm)
    bLoad: TButton;
    OpenDialog1: TOpenDialog;
    Layout1: TLayout;
    tbZoom: TTrackBar;
    sb: TScrollBox;
    lSize: TLayout;
    bEnterSVGText: TButton;
    Edit2: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    lZoom: TLayout;
    XMLDocument1: TXMLDocument;
    SVGDelphiXMLDoc1: TSVGDelphiXMLDoc;
    SVGAgent1: TSVGAgent;
    bCopySelected: TButton;
    tvSVG: TTreeView;
    Splitter1: TSplitter;
    cbAnalyzeStructure: TCheckBox;
    eStats: TEdit;
    Label3: TLabel;
    bBringToFront: TButton;
    procedure FormCreate(Sender: TObject);
    procedure tbZoomChange(Sender: TObject);
    procedure sbClick(Sender: TObject);
    procedure bEnterSVGTextClick(Sender: TObject);
    procedure bLoadClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; var KeyChar: Char;
      Shift: TShiftState);
    procedure sbMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure sbMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Single);
    procedure sbMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure bCopySelectedClick(Sender: TObject);
    procedure tvSVGClick(Sender: TObject);
    procedure cbAnalyzeStructureClick(Sender: TObject);
    procedure bBringToFrontClick(Sender: TObject);
  private
    FZoom : single;
    FSVGSelection : TSVGSelection;
    FCurrentSVGControl : TSVGGroup; // Current SVG Control
    FCurrentSVGObject : TSVGObject; // Current child object of SVG Control
    FDX, FDY : single;
  public
    procedure CalcLayoutDimensions;

    procedure UpdateTreeview( aSVG : TSVGGroup);

    procedure CopySVG;

    function  CreateSVGGroup: TSVGGroup;
    procedure CreateSVGFromStrings( aSvgText: TStrings);

    procedure SVGMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Single);
    procedure SVGMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Single);
    procedure SVGMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Single);
  end;

var
  frmSVGTest: TfrmSVGTest;

implementation

{$R *.fmx}

uses Unit2;

function RealignChildren( aControl : TFmxObject; dx, dy : single): TRectF;
var i : integer;
    c : TControl;
begin
  // Calc max size of a parent control
  Result.Left := 100000;
  Result.Right := 0;
  Result.Top := 100000;
  Result.Bottom := 0;
  for i := 0 to aControl.ChildrenCount - 1 do begin
    if aControl.Children[i] is TControl then begin
      c := (aControl.Children[i] as TControl);
      c.Position.X := c.Position.X + dx;
      c.Position.Y := c.Position.Y + dy;

      if c.Position.X < Result.Left then
        Result.Left := c.Position.X;

      if c.Position.Y < Result.Top then
        Result.Top := c.Position.Y;

      if c.Position.X + c.Width > Result.Right then
        Result.Right := c.Position.X + c.Width;

      if c.Position.Y + c.Height > Result.Bottom then
        Result.Bottom := c.Position.Y + c.Height;
    end;
  end;
end;

procedure TfrmSVGTest.CalcLayoutDimensions;
var R : TRectF;
begin
  // Calc max size of lZoom layout
  R := RealignChildren( lZoom, 0, 0);

  lZoom.Width := R.Right;
  lZoom.Height := R.Bottom;
  lSize.Width := lZoom.Width * tbZoom.Value;
  lSize.Height := lZoom.Height * tbZoom.Value;
end;

procedure TfrmSVGTest.FormCreate(Sender: TObject);
begin
  FormatSettings.DecimalSeparator := '.';
  FZoom := tbZoom.Value;

  FSVGSelection := TSVGSelection.Create(self);
  FSVGSelection.Parent := lZoom;
  FSVGSelection.Position.X := 0;
  FSVGSelection.Position.Y := 0;
  FSVGSelection.Width := sb.Width / tbZoom.Value;
  FSVGSelection.Height := sb.Height  / tbZoom.Value;
  FSVGSelection.Active := False;

  tbZoomChange(self);
end;

procedure TfrmSVGTest.tbZoomChange(Sender: TObject);
var mx, my : single;
    i : integer;
begin
  // Calc middle of currently visible rectangle of scrollbox

  mx := (sb.ViewportPosition.X + sb.Width/2) / FZoom;
  my := (sb.ViewportPosition.Y + sb.Height/2) / FZoom;

  FZoom := tbZoom.Value;

  // Set zoom of lZoom layout
  lZoom.Scale.X := FZoom;
  lZoom.Scale.Y := FZoom;

  // Calc new size of lSize layout
  lSize.Width := lZoom.Width * FZoom;
  lSize.Height := lZoom.Height * FZoom;

  // Adjust scrollbox to keep centered on original mid point
  sb.ViewportPosition := PointF( mx * FZoom - (sb.Width/2), my * FZoom - (sb.Height/2));
end;

procedure TfrmSVGTest.cbAnalyzeStructureClick(Sender: TObject);
begin
  if assigned(FCurrentSVGControl) then
    UpdateTreeView(FCurrentSVGControl);
end;


procedure TfrmSVGTest.tvSVGClick(Sender: TObject);
begin
  // Show/hide selected SVG part in treeview
  tvSVG.BeginUpdate;
  try
    if assigned(tvSVG.Selected) and (tvSVG.Selected is TSVGTreeViewItem) then begin
      (tvSVG.Selected as TSVGTreeViewItem).ObjectVisible := not (tvSVG.Selected as TSVGTreeViewItem).ObjectVisible;
      (tvSVG.Selected as TSVGTreeViewItem).SVGParent.Repaint;
    end;
  finally
    tvSVG.EndUpdate;
  end;
end;

procedure TfrmSVGTest.UpdateTreeview( aSVG: TSVGGroup);

  Procedure AddNode( ParentNode : TFMXObject; SVGObject : TSVGObject);
  var n : TSVGTreeViewItem;
      i : integer;
      SVGGroup : TSVGGroup;
      R : TRectF;
  begin
    n := TSVGTreeViewItem.Create(self);
    n.Parent := ParentNode;
    n.SVGObject := SVGObject;
    n.SVGParent := aSVG;
    if SVGObject is TSVGGroup then begin
      SVGGroup := SVGObject as TSVGGroup;
      for i := 0 to SVGGroup.Count - 1 do
        AddNode(n, SVGGroup.Item[i]);
    end;
  end;

begin
  if not cbAnalyzeStructure.IsPressed then
    exit;


  tvSVG.BeginUpdate;
  try
    tvSVG.Clear;
    AddNode( tvSVG, aSVG);
    //tvSVG.ExpandAll; Slow!
  finally
    tvSVG.EndUpdate;
  end;
  tvSVG.Repaint;
end;

function TfrmSVGTest.CreateSVGGroup: TSVGGroup;
begin
  // Create new SVG group object
  Result := TSVGGroup.Create(self);
  Result.Parent := lZoom;
  Result.Position.X := 50;
  Result.Position.Y := 50;
  Result.Width := 2;
  Result.Height := 2;
  Result.HitTest := True;

  Result.OnMouseDown := SVGMouseDown;
  Result.OnMouseMove := SVGMouseMove;
  Result.OnMouseUp := SVGMouseUp;
end;

procedure TfrmSVGTest.CreateSVGFromStrings( aSvgText: TStrings);
var SVGGroup : TSVGGroup;
begin
  // Create an SVG control object from strings
  SVGDelphiXMLDoc1.Active := False;
{$IFDEF MSWINDOWS}
  XMLDocument1.DOMVendor := GetDOMVendor('MSXML'); // If windows
  XMLDocument1.ParseOptions := [poPreserveWhiteSpace];
{$ENDIF}

{$IFDEF OSX}
  XMLDocument1.DOMVendor := GetDOMVendor('ADOM XML v4'); // If OSX
  XMLDocument1.ParseOptions := [poPreserveWhiteSpace];
{$ENDIF}
  XMLDocument1.XML.Text := aSVGText.Text;

  // This sets the XMLDocument to active but also compiles an index of SVG-id's
  SVGDelphiXMLDoc1.Active := True;

  SVGGroup := CreateSVGGroup;

  // Start pasing from root element of SVG
  SVGAgent1.ParseSVG('', SVGGroup, CreateUnityMatrix);

  eStats.Text := 'Groups: ' + IntToStr(SVGAgent1.GroupCount)
               + ', paths: ' + IntToStr(SVGAgent1.PathCount)
               + ', spans: ' + IntToStr(SVGAgent1.SpanCount)
               + ', text: ' + IntToStr(SVGAgent1.TextCount)
               + ', effects: ' + IntToStr(SVGAgent1.EffectCount);

  CalcLayoutDimensions;
end;

procedure TfrmSVGTest.bLoadClick(Sender: TObject);
var SVGGroup: TSVGGroup;
    Clock : TSVGAnimatedClock;
    filename : string;
begin
  if OpenDialog1.Execute then begin

    // Load an SVG XML from file
    SVGDelphiXMLDoc1.Active := False;
{$IFDEF MSWINDOWS}
    XMLDocument1.DOMVendor := GetDOMVendor('MSXML'); // If windows
    XMLDocument1.ParseOptions := [poPreserveWhiteSpace];
{$ENDIF}

{$IFDEF OSX}
    XMLDocument1.DOMVendor := GetDOMVendor('ADOM XML v4'); // If OSX
    XMLDocument1.ParseOptions := [poPreserveWhiteSpace];
{$ENDIF}
    XMLDocument1.LoadFromFile( OpenDialog1.FileName);

    // This sets the XMLDocument to active but also compiles an index of SVG-id's
    SVGDelphiXMLDoc1.Active := True;

    // Check if  it's the animated clock
    filename := ExtractFilename(openDialog1.FileName);
    if Lowercase(filename) = 'animated-clock.svg' then begin
      Clock := TSVGAnimatedClock.Create( self, 'animated_clock', nil, CreateUnityMatrix, CreateUnityMatrix);

      // Start pasing from root element of SVG
      SVGAgent1.ParseSVG( '', CLock, CreateUnityMatrix);
      Clock.Position.X := 50;
      Clock.Position.Y := 50;
      Clock.Parent := lZoom;

      Clock.OnMouseDown := SVGMouseDown;
      Clock.OnMouseMove := SVGMouseMove;
      Clock.OnMouseUp := SVGMouseUp;
      Clock.Init;
      Clock.HitTest := True;

    end else begin
      SVGGroup := CreateSVGGroup;
      // Start pasing from root element of SVG
      SVGAgent1.ParseSVG('', SVGGroup, CreateUnityMatrix);
    end;

    eStats.Text := 'Groups: ' + IntToStr(SVGAgent1.GroupCount)
                 + ', paths: ' + IntToStr(SVGAgent1.PathCount)
                 + ', spans: ' + IntToStr(SVGAgent1.SpanCount)
                 + ', text: ' + IntToStr(SVGAgent1.TextCount)
                 + ', effects: ' + IntToStr(SVGAgent1.EffectCount);

    CalcLayoutDimensions;
  end;
end;

procedure TfrmSVGTest.bBringToFrontClick(Sender: TObject);
var i : integer;
    ObjectCopy : TSVGGroup;
begin
  if assigned(FSVGSelection) then begin
    for i := 0 to FSVGSelection.FSelList.Count - 1 do
      if FSVGSelection.FSelList[i] is TSVGGroup then begin
        (FSVGSelection.FSelList[i] as TSVGGroup).BringToFront;
        (FSVGSelection.FSelList[i] as TSVGGroup).Repaint;
      end;
  end;
end;

procedure TfrmSVGTest.bCopySelectedClick(Sender: TObject);
begin
  CopySVG;
end;

procedure TfrmSVGTest.bEnterSVGTextClick(Sender: TObject);
begin
  Form2.Show;
end;

procedure TfrmSVGTest.sbClick(Sender: TObject);
begin
  FSVGSelection.UnSelectAll;
end;

procedure TfrmSVGTest.CopySVG;
var i : integer;
    ObjectCopy : TSVGGroup;
begin
  if assigned(FSVGSelection) then begin
    for i := 0 to FSVGSelection.FSelList.Count - 1 do begin
      ObjectCopy := TSVGObject(FSVGSelection.FSelList[i]).CreateCopy(self, nil) as TSVGGroup;

      ObjectCopy.Parent := lZoom;
      ObjectCopy.Position.X := 250;
      ObjectCopy.Position.Y := 250;
      ObjectCopy.HitTest := True;

      ObjectCopy.OnMouseDown := SVGMouseDown;
      ObjectCopy.OnMouseMove := SVGMouseMove;
      ObjectCopy.OnMouseUp := SVGMouseUp;

      FSVGSelection.AddToSelection( ObjectCopy);
    end;
  end;
end;

procedure TfrmSVGTest.FormKeyDown(Sender: TObject; var Key: Word;
  var KeyChar: Char; Shift: TShiftState);
begin
  if Key = vkDelete then begin
    if assigned(FCurrentSVGControl) then begin
      FreeAndNil(FCurrentSVGControl);
      FSVGSelection.UnselectAll;
    end;
  end;
end;

// Hittest is on for SVG Objects and scrollbox, so mouseevents are handled
// for both. Coordinates are calculated relative to the lZoom layout

procedure TfrmSVGTest.sbMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
var P: TPointF;
begin
  // Scrollbox Mousedown

  if Button = TMouseButton.mbLeft then begin

    // Get startpoint in lZoom layout
    P := PointF(X, Y);
    P := lZoom.AbsoluteToLocal(P);

    FDX := P.X;
    FDY := P.Y;
    FCurrentSVGObject := nil;
    FCurrentSVGControl := nil;
  end;
end;

procedure TfrmSVGTest.SVGMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
var P: TPointF;
begin
  // SVG Object mousedown

  if not(Sender is TSVGObject) then
    exit;

  if Button = TMouseButton.mbLeft then begin

    // Get startpoint in lZoom layout
    P := (Sender as TControl).LocalToAbsolute(PointF(X, Y));
    P := lZoom.AbsoluteToLocal(P);

    // Make the SVG Control the current control
    if Sender is TSVGGroup then
       FCurrentSVGControl := Sender as TSVGGroup;

    FDX := P.X;
    FDY := P.Y;
    FCurrentSVGObject := nil;
  end;
end;

procedure TfrmSVGTest.sbMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Single);
var dx, dy : single;
    Clock : TSVGAnimatedClock;
    P: TPointF;
begin
  // Scrollbox mousemove

  P := PointF(X, Y);
  P := lZoom.AbsoluteToLocal(P);

  if (ssLeft in Shift) then begin

    if assigned(FCurrentSVGObject) then begin
      if assigned(FCurrentSVGControl) then begin

        if (FCurrentSVGControl is TSVGAnimatedClock) then begin
          // Move the clock hands

          Clock := FCurrentSVGControl as TSVGAnimatedClock;

          dx := P.X - FDX;
          dy := P.Y - FDY;
          FDX := P.X;
          FDY := P.Y;

          Clock.ProcessMouseMove( FCurrentSVGObject, Shift, dx, dy);
        end;
      end;
    end else
      if FSVGSelection.Active then begin

        // If a group selection is active, move the selection
        FSVGSelection.Position.X := P.X - FDX;
        FSVGSelection.Position.Y := P.Y - FDY;

      end;
  end;
end;

procedure TfrmSVGTest.SVGMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Single);
var
  P: TPointF;
  Clock : TSVGAnimatedClock;
  dx, dy : single;
  MouseOverObject : TSVGObject;
begin
  // SVG Object mousemove

  if not(Sender is TSVGObject) then
    exit;

  MouseOverObject := Sender as TSVGObject;
  P := MouseOverObject.LocalToAbsolute(PointF(X, Y));
  P := lZoom.AbsoluteToLocal(P);

  if (ssLeft in Shift) then begin

    if assigned(FCurrentSVGObject) then begin
      if assigned(FCurrentSVGControl) then begin

        if (FCurrentSVGControl is TSVGAnimatedClock) then begin
          // Move the clock hands
          Clock := FCurrentSVGControl as TSVGAnimatedClock;

          dx := P.X - FDX;
          dy := P.Y - FDY;
          FDX := P.X;
          FDY := P.Y;

          Clock.ProcessMouseMove( FCurrentSVGObject, Shift, dx, dy);
        end;

      end;
    end else

      if FSVGSelection.Active then begin
        // If a group selection is active, move the selection

        FSVGSelection.Position.X := P.X - FDX;
        FSVGSelection.Position.Y := P.Y - FDY;

      end else begin
        if (abs(P.X-FDX)>0.1) or (abs(P.Y-FDY)>0.1) then begin
          if (MouseOverObject is TSVGGroup) then begin

            FCurrentSVGControl := MouseOverObject as TSVGGroup;

            if (not assigned(FCurrentSVGObject)) then begin
              // Mouse is moved over the inside of a SVG Control
              // Check which child control the mouse is moving over and
              // make that child control the current one...

              // Only child objects with Hittest = True are seen.

              // If the control is constructed with SVG agent property "Parenting"
              // is True, you wont need this, see "HandMouseMove" of TAnimatedClock

              FCurrentSVGObject := FCurrentSVGControl.ObjectAtPt( PointF(X , Y));
              if assigned(FCurrentSVGObject) then begin
                Edit2.Text := FCurrentSVGObject.ID;
                // If the parentcontrol isn't currently selected, select it here
                if not FSVGSelection.Selected(FCurrentSVGControl) then begin
                  if ssShift in Shift then
                    FSVGSelection.AddToSelection( FCurrentSVGControl)
                  else begin
                    FSVGSelection.SelectObject( FCurrentSVGControl);
                    UpdateTreeView( FCurrentSVGControl);
                  end;
                end;

              end else begin
                // The mouse is moved with left button down on a SVG Control and
                // not over a child with Hittest = True
                // Add the control to the selection and move the selection

                // If the control wasn't selected yet, then select it here
                if not FSVGSelection.Selected(FCurrentSVGControl) then begin
                  if ssShift in Shift then
                    FSVGSelection.AddToSelection( FCurrentSVGControl)
                  else begin
                    FSVGSelection.SelectObject( FCurrentSVGControl);
                    UpdateTreeView( FCurrentSVGControl);
                  end;
                end;

                // When the selction is activated, bitmaps are created and
                // placed on the selection layout. The selected SVG controls
                // are set to Visible = False, until the selection is deactivated.

                // When zoomed-in, this will give an ugly result, because the
                // bitmaps are zoomed. You could change this by for example draw
                // ouline rectangles in stead of bitmap copies on the selection
                // layout.

                FSVGSelection.Position.X := 0;
                FSVGSelection.Position.Y := 0;
                FSVGSelection.Width := sb.Width / tbZoom.Value;
                FSVGSelection.Height := sb.Height  / tbZoom.Value;
                FSVGSelection.Active := True;

                FDX := P.X;
                FDY := P.Y;
              end;
            end;
          end;
        end;
      end;
  end;
end;

procedure TfrmSVGTest.sbMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
  // Scrollbox mouseup

  if FSVGSelection.Active then begin
    // Deactivate the selcetion

    FSVGSelection.Active := False;
    CalcLayoutDimensions;
  end;
end;

procedure TfrmSVGTest.SVGMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
  // SVG Object mouseup
  if (Sender is TSVGGroup) and not(assigned(FCurrentSVGControl)) then begin

    // Add the control to the selection
    FCurrentSVGControl := Sender as TSVGGroup;
    FCurrentSVGObject := nil;

    if ssShift in Shift then
      FSVGSelection.AddToSelection( FCurrentSVGControl)
    else begin
      FSVGSelection.SelectObject( FCurrentSVGControl);
      UpdateTreeView( FCurrentSVGControl);
    end;
    FCurrentSVGControl := nil;
  end else
    if FSVGSelection.Active then begin
      // Deactivate the selection
      FSVGSelection.Active := False;
      CalcLayoutDimensions;
    end;
end;


//==============================================================================
//
//                              TSelectionLayout
//
//==============================================================================

constructor TSVGSelection.Create(AOwner: TComponent);
begin
  inherited;

  HitTest := False;
  FSelList := TObjectList.Create(False);
end;

destructor TSVGSelection.Destroy;
begin
  FSelList.Free;

  inherited;
end;

procedure TSVGSelection.AddToSelection(aSVGGroup: TSVGGroup);
var i : integer;
begin
  i := FSelList.IndexOf( aSVGGroup);
  if i <> -1 then begin
    FSelList.Delete(i);
    aSVGGroup.Fill.Kind := TBrushKind.bkNone;
  end else begin
    FSelList.Add(aSVGGroup);
    aSVGGroup.Fill.Color := claYellow;
    aSVGGroup.Fill.Kind := TBrushKind.bkSolid;
  end;
end;

procedure TSVGSelection.UnSelectAll;
var SVGGroup : TSVGGroup;
begin
  while FSelList.Count > 0 do begin
    SVGGroup := FSelList[0] as TSVGGroup;
    SVGGroup.Fill.Kind := TBrushKind.bkNone;
    FSelList.Delete(0);
  end;
end;

function TSVGSelection.Selected(aSVGGroup: TSVGGroup): boolean;
begin
  Result := FSelList.IndexOf( aSVGGroup) <> -1;
end;

procedure TSVGSelection.SelectObject(aSVGGroup: TSVGGroup);
var i : integer;
begin
  i := FSelList.IndexOf( aSVGGroup);
  if i <> -1 then begin
    UnSelectAll;
  end else begin
    UnSelectall;
    FSelList.Add(aSVGGroup);
    aSVGGroup.Fill.Color := claYellow;
    aSVGGroup.Fill.Kind := TBrushKind.bkSolid;
  end;
end;

function TSVGSelection.GetActive: boolean;
begin
  Result := Visible;
end;

procedure TSVGSelection.SetActive(const aValue: boolean);
var i : integer;
    Control : TControl;
    Image : TImage;
    Bitmap : TBitmap;
begin
  if aValue <> Visible then begin
    if aValue then begin
      // Activate the selection layout
      DeleteChildren;

      // Make a screenshot of all selected objects and put on selection layout
      for i := 0 to FSelList.Count - 1 do begin
        Control := (FSelList[i] as TControl);
        Bitmap := Control.MakeScreenshot;
        try
          Image := TImage.Create( self);
          Image.Bitmap.Assign( Bitmap);
          Image.HitTest := False;
          Image.Position.Assign( Control.Position);
          Image.Width := Control.Width;
          Image.Height := Control.Height;
          Image.Parent := self;
        finally
          BitMap.Free;
        end;

        // Hide the original objects
        Control.Visible := False;
      end;
    end else begin
      // Deactivate the selection layout

      // Write new positions in selected controls and make visible again
      for i := 0 to FSelList.Count - 1 do begin
        (FSelList[i] as TControl).Position.X := (FSelList[i] as TControl).Position.X + Position.X;
        (FSelList[i] as TControl).Position.Y := (FSelList[i] as TControl).Position.Y + Position.Y;
        (FSelList[i] as TControl).Visible := True;
        (FSelList[i] as TControl).Hittest := True;
      end;
      DeleteChildren;
    end;
    Visible := aValue;
  end;
end;

//==============================================================================
//
//                               TSVGAnimatedClock
//
//==============================================================================


constructor TSVGAnimatedClock.Create(AOwner: TComponent);
begin
  inherited;

  FTimer := TTimer.Create(self);
  FTimer.Interval := 1000;
  FTimer.Enabled := False;
  FTimer.OnTimer := SetTime;
end;

constructor TSVGAnimatedClock.Create(AOwner: TComponent; aId : string; aSVGParent : TSVGGroup; aCTM, aUserMatrix : TMatrix);
begin
  inherited Create(AOwner);

  FTimer := TTimer.Create(self);
  FTimer.Interval := 1000;
  FTimer.Enabled := False;
  FTimer.OnTimer := SetTime;
end;

destructor TSVGAnimatedClock.Destroy;
begin
  inherited;
end;

function TSVGAnimatedClock.CreateCopy(AOwner : TComponent; aSVGParent : TSVGGroup): TSVGObject;
begin
  Result := TSVGAnimatedClock.Create(AOwner);
  Result.Assign(self);
end;

procedure TSVGAnimatedClock.Assign(Source: TPersistent);
begin
  inherited;
  if Source is TSVGAnimatedClock then begin
    Init;
  end;
end;

procedure TSVGAnimatedClock.Init;
begin
  // Set pointers to interactive/animated SVG parts
  FGearSeconds := FindObject( 'RotSeconds') as TSVGGroup;
  FGear7_5 := FindObject( 'RotGear7_5') as TSVGGroup;
  FGearMinutes := FindObject( 'RotMinutes') as TSVGGroup;
  FGear180 := FindObject( 'RotGear180') as TSVGGroup;
  FGearHours := FindObject( 'RotHours') as TSVGGroup;

  // The interactive SVG objects should have Hittest = TRUE otherwise they
  // are ignored by TSVGGroup.ObjectAt() or mouseevents
  FHandSecond := FindObject( 'RotSecondHand') as TSVGGroup;
  FPathHandSecond := FindObject( 'SecondHandPath') as TSVGPath;
  FPathHandSecond.HitTest := True;

  FHandMinute := FindObject( 'RotMinuteHand') as TSVGGroup;
  FPathHandMinute := FindObject( 'MinuteHandPath') as TSVGPath;
  FPathHandMinute.HitTest := True;

  FHandHour:= FindObject( 'RotHourHand') as TSVGGroup;
  FPathHandHour := FindObject( 'HourHandPath') as TSVGPath;
  FPathHandHour.HitTest := True;

  // If the SVG Control is constructed with the SVG Agent property ""Parenting"
  // set to True, we have to adjust the rotation center of the clock hands. Because
  // Pathdata is made relative to the parent object. On the plus side, we can
  // use event handling of the objects.

  FHandSecond.RotationCenter.Y := 0.74; // If NormalParenting
  FHandMinute.RotationCenter.Y := 0.74; // If NormalParenting
  FHandHour.RotationCenter.Y := 0.74; // If NormalParenting

  FPathHandSecond.OnMouseDown := HandMouseDown;
  FPathHandMinute.OnMouseDown := HandMouseDown;
  FPathHandHour.OnMouseDown := HandMouseDown;
  FPathHandHour.OnMouseMove := OnMouseMove;
  FPathHandHour.OnMouseUp := OnMouseUp;

  FTimer.Enabled := True;
end;

procedure TSVGAnimatedClock.ProcessMouseMove(aSVGObject: TSVGObject;
  Shift: TShiftState; dX, dY: Single);
begin
  if aSVGObject.ID = 'SecondHandPath' then begin
    AddDeltaSec( -dy / 23);
  end;

  if aSVGObject.ID = 'MinuteHandPath' then begin
    AddDeltaMin( -dy / 23);
  end;

  if aSVGObject.ID = 'HourHandPath' then begin
    AddDeltaHour( -dy / 23);
  end;
end;

procedure TSVGAnimatedClock.AddDeltaSec( dm : single);
begin
  FTimer.Enabled := False;

  if FSec + dm > 60 then begin
    AddDeltaMin(1);
    FSec := trunc(FSec + dm - 60);
  end else
    if FSec + dm < 0 then begin
      AddDeltaMin(-1);
      FSec := FSec + dm + 60;
    end else
      FSec := FSec + dm;

  Update;
end;

procedure TSVGAnimatedClock.AddDeltaMin( dm : single);
begin
  FTimer.Enabled := False;

  if FMin + dm > 60 then begin
    AddDeltaHour(1);
    FMin := trunc(Fmin + dm - 60);
  end else
    if FMin + dm < 0 then begin
      AddDeltaHour(-1);
      FMin := FMin + dm + 60;
    end else
      FMin := FMin + dm;

  Update;
end;

procedure TSVGAnimatedClock.AddDeltaHour( dm : single);
begin
  FTimer.Enabled := False;

  if FHour + dm > 24 then begin
    FHour := trunc(FHour + dm - 24);
  end else
    if FHour + dm < 0 then begin
      FHour := FHour + dm + 24;
    end else
      FHour := FHour + dm;

  Update;
end;

procedure TSVGAnimatedClock.Update;
var a_sec, a_7_5, a_min, a_180, a_hours : single;
    tot_s : single;
begin
  tot_s := FMin * 60 + FHour * 3600 + FSec;

  a_sec   :=   (tot_s * 6) - 360 * trunc( tot_s * 6 / 360);
  a_7_5   := -((tot_s * 6 / 7.5) - 360 * trunc(tot_s * 6 / 7.5 / 360));
  a_min   :=   (tot_s * 6 /  60) - 360 * trunc(tot_s * 6 /  60 / 360);
  a_180   := -((tot_s * 6 / 180) - 360 * trunc(tot_s * 6 / 180 / 360));
  a_hours :=   (tot_s * 6 / 720) - 360 * trunc(tot_s * 6 / 720 / 360);

  SetLayoutAngle( FGearSeconds, a_sec);
  SetLayoutAngle( FGear7_5, a_7_5);
  SetLayoutAngle( FGearMinutes, a_min);
  SetLayoutAngle( FGear180, a_180);
  SetLayoutAngle( FGearHours, a_hours);
  SetLayoutAngle( FHandSecond, a_sec);
  SetLayoutAngle( FHandMinute, a_min);
  SetLayoutAngle( FHandHour, a_hours);

  Repaint;
end;

procedure TSVGAnimatedClock.SetLayoutAngle( aLayout: TSVGGroup; aAngle : single);
begin
  // Set the new position of a SVG group object
  if frmSVGTest.SVGAgent1.Parenting then
    aLayout.RotationAngle := aAngle
  else
    aLayout.Transform(1,1,0,0, aAngle*PI/180, 0,0);
end;

procedure TSVGAnimatedClock.SetTime( Sender : TObject);
var h, m, s, ms : word;
begin
  DecodeTime( Now, h, m, s, ms);
  FHour := h;
  FMin := m;
  FSec := s;
  Update;
end;

procedure TSVGAnimatedClock.HandMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Single);
var P: TPointF;
begin
 // MouseDown( Button, Shift, X, Y);
  if assigned(OnMouseDown) then
    OnMouseDown( Sender, Button, Shift, X, Y);

  if Button = TMouseButton.mbLeft then begin
    frmSVGTest.FCurrentSVGObject := Sender as TSVGObject;
    frmSVGTest.FCurrentSVGControl := Self;
  end;
end;

//==============================================================================
//
//                               TSVGTreeViewItem
//
//==============================================================================

constructor TSVGTreeViewItem.Create(AOwner: TComponent);
begin
  inherited;
end;

destructor TSVGTreeViewItem.Destroy;
begin
  inherited;
end;

function TSVGTreeViewItem.GetObjectVisible: boolean;
begin
  Result := FSVGObject.Visible;
end;

procedure TSVGTreeViewItem.SetObjectVisible(aValue: boolean);
begin
  FSVGObject.Visible := aValue;
  SetSVGText;
end;

procedure TSVGTreeViewItem.SetSVGObject(aValue: TSVGObject);
begin
  FSVGObject := aValue;
  SetSVGText;
end;

procedure TSVGTreeViewItem.SetSVGText;
var Temp : string;
begin
  if FSVGObject is TSVGPath then
    Temp := 'path:';

  if FSVGObject is TSVGGroup then
    Temp := 'group:';

  if FSVGObject is TSVGText then
    Temp := 'text:';

  Temp := Temp + ' ' + FSVGObject.ID;

  if not FSVGObject.Visible then begin
    Temp := Temp + ' (hidden)';
  end;
  Text := Temp;
end;

// Prevent the 'DTD is prohibited' error.
{$IFDEF MSWINDOWS}
initialization
{$IFDEF VER230}
  MSXML6_ProhibitDTD := False;
{$ELSE}
   MSXMLDOMDocumentFactory.AddDOMProperty('ProhibitDTD', False);
{$ENDIF}
{$ENDIF}
end.
