unit Unit1;

// iOS demo

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  System.IOUtils,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Dialogs, Xml.xmldom, Xml.XMLIntf,
  Xml.XMLDoc, Xml.adomxmldom,
  BVE.SVGControl, BVE.SVGXMLWrapperDelphi, FMX.StdCtrls, FMX.Edit, FMX.ListBox,
  FMX.Layouts, FMX.TabControl;

type
  TForm1 = class(TForm)
    XMLDocument1: TXMLDocument;
    tbZoom: TTrackBar;
    TabControl1: TTabControl;
    TabItem1: TTabItem;
    TabItem2: TTabItem;
    sb: TScrollBox;
    Layout1: TLayout;
    ListBox1: TListBox;
    lSize: TLayout;
    lZoom: TLayout;
    procedure FormCreate(Sender: TObject);
    procedure tbZoomChange(Sender: TObject);
    procedure TabItem2Click(Sender: TObject);
  private
    { Private declarations }
    FZoom : single;
    FSVGPath : string;
  public
    { Public declarations }
    FXMLWrapper : TSVGDelphiXMLDoc;
    FSVGAgent : TSVGAgent;
    FSVGControl : TSVGControl;
    procedure ReadDir;
  end;

var
  Form1: TForm1;

implementation

{$R *.fmx}


procedure TForm1.FormCreate(Sender: TObject);
begin
  GlobalUseDirect2D := true;
  GlobalUseHWEffects := false;


  FXMLWrapper := TSVGDelphiXMLDoc.Create(self);
  FSVGAgent := TSVGAgent.Create(self);
  FSVGControl := TSVGControl.Create(self);

  FXMLWrapper.XMLDoc := XMLDocument1;
  FSVGAgent.SVGDoc := FXMLWrapper;
  FSVGAgent.Parenting := False;
  FSVGControl.SVGAgent := FSVGAgent;
  FSVGControl.Parent := lZoom;
  FSVGControl.HitTest := False;

  FZoom := tbZoom.Value;

  // If you add SVG files using Project->Deployment on remote path Startup\Documents\
  // they will be found using this path:

  FSVGPath := GetHomePath + PathDelim + 'Documents' + PathDelim;
  ReadDir;
end;

procedure TForm1.ReadDir;
var sr : TSearchRec;
    test : string;
begin
 if FindFirst( FSVGPath + '*.svg', faNormal, sr) = 0 then
   try
     repeat
       Listbox1.Items.Add(sr.Name);
     until FindNext(sr)<>0;
   finally
     FindClose(sr);
   end;
end;

procedure TForm1.TabItem2Click(Sender: TObject);
var R : TRectF;
begin
  if Listbox1.ItemIndex = -1 then
    exit;

  FXMLWrapper.Active := False;
  XMLDocument1.FileName := FSVGPath + ListBox1.Items[Listbox1.ItemIndex];
  FXMLWrapper.Active := True;

  lZoom.Width := FSVGControl.Position.X + FSVGControl.Width;
  lZoom.Height := FSVGControl.Position.Y + FSVGControl.Height;
  lSize.Width := lZoom.Width * tbZoom.Value;
  lSize.Height := lZoom.Height * tbZoom.Value;
end;

procedure TForm1.tbZoomChange(Sender: TObject);
var mx, my : single;
    i : integer;
begin
  // Calc middle of currently visible rectangle of scrollbox

  mx := (sb.ViewportPosition.X + sb.Width/2) / FZoom;
  my := (sb.ViewportPosition.Y + sb.Height/2) / FZoom;

  FZoom := tbZoom.Value;

  // Set zoom of lZoom layout
  lZoom.Scale.X := FZoom;
  lZoom.Scale.Y := FZoom;

  // Calc new size of lSize layout
  lSize.Width := lZoom.Width * FZoom;
  lSize.Height := lZoom.Height * FZoom;

  // Adjust scrollbox to keep centered on original mid point
  sb.ViewportPosition := PointF( mx * FZoom - (sb.Width/2), my * FZoom - (sb.Height/2));
end;

end.
